package models

type BaseFromKpler struct {
	Date          string
	Value         float64
	PeriodEndDate string
}

  // withIntraCountry := false
  // withIntraRegion := true
  // withForecast := true
  // withFreightView := false
  // withProductEstimation := false
  type KplerFlowDataLibReq struct {
	Products string `description:"产品名称"`
	FromZones string `description:"来源区域ID，对应Location" ` 
	ToZones string `description:"流向区域ID"`
	Split string `description:"拆分类型"`
	FlowDirection string `description:"流向，对应periodicity：export/import"`
	Granularity string `description:"粒度: daily/weekly/monthly/yearly"`
	Unit string `description:"单位"`
	FromCountries string `description:"来源国家"`
	ToCountries string `description:"流向国家"`
	FromInstallations string `description:"来源安装"`
	ToInstallations string `description:"流向安装"`
	OnlyRealized string `description:"是否只查询已实现数据"`
	VesselTypes string `description:"船型"`
	VesselTypesAlt string `description:"船型"`
    WithIntraCountry string `description:"是否查询同国数据"`
    WithIntraRegion string `description:"是否查询同区域数据"`
    WithForecast string `description:"是否查询预测数据"`
    WithFreightView string `description:"是否查询运费数据"`
    WithProductEstimation string `description:"是否查询产品估算数据"`
    StartDate string `description:"开始日期"`
    EndDate string `description:"结束日期"`
}

type KplerFlowDataResp struct {
	ApiQueryUrl string
	List []KplerFlowData 
}
type KplerFlowData struct {
	SplitItem string 
	IndexData []KplerBaseExcelData 
}

type KplerBaseExcelData struct {
	DataTime string
	Value    string
}

//bodystr := "Id (Product);Name;Type (Product);Family;Family Id;Group;Group Id;Product;Product Id;Grade;Grade Id;Density (Product);Density Unit;Energy Density;Energy Density Unit;Expansion Ratio
type KplerProduct struct {
	Id string
	Name string
	Type string
	Family string
	FamilyId string
	Group string
	GroupId string
	Product string
	ProductId string
	Grade string
	GradeId string
	Density string
	DensityUnit string
	EnergyDensity string
	EnergyDensityUnit string
	ExpansionRatio string
}

// Ancestor Id;Ancestor Name;Ancestor Type;Descendant Id;Descendant Name;Descendant Type
type KplerZone struct {
	AncestorId string
	AncestorName string
	AncestorType string
	DescendantId string
	DescendantName string
	DescendantType string
}

type KplerZoneDataLibReq struct {
	Token string `description:"token"`
	AncestorName string `description:"祖先名称"`
	DescendantType string `description:"子类型"`
}

// HandleKplerExcelData Kpler的excel数据
type HandleKplerExcelData struct {
	ClassifyName       string `description:"指标目录"`
	ParentClassifyName string `description:"父级指标目录"`
	ClassifySort       int    `description:"指标目录排序号"`
	IndexName          string `description:"指标名称"`
	IndexCode          string `description:"指标编码"`
	Unit               string `description:"单位"`
	Sort               int    `description:"排序号"`
	Frequency          string `description:"频度"`
	ProductNames       string `description:"产品名称"`
	FromZoneNames      string `description:"区域名称"`
	ToZoneNames        string `description:"区域名称"`
	FlowDirection      string `description:"流向"`
	Granularity        string `description:"粒度"`
	Split              string `description:"拆分类型"`
	SplitName          string `description:"拆分类型名称"`
	ExcelQueryUrl      string `description:"Excel查询URL"`
	ExcelDataMap       map[string]string
}

type HandleKplerExcelDataReq struct {
	List         []*HandleKplerExcelData
	TerminalCode string `description:"编码"`
}

type KplerProductLibReq struct {
	AncestorFamilyIds string `description:"祖先家族ID"`
	AncestorFamilyNames string `description:"祖先家族名称"`
	AncestorGroupIds string `description:"祖先组ID"`
	AncestorGroupNames string `description:"祖先组名称"`
	AncestorProductIds string `description:"祖先产品ID"`
	AncestorProductNames string `description:"祖先产品名称"`
	AncestorGradeIds string `description:"祖先等级ID"`
	AncestorGradeNames string `description:"祖先等级名称"`
	Products string `description:"产品"`
	ProductIds string `description:"产品ID"`
}
	