package models

// KplerLocation represents a location with ID and name
type KplerExcelLocation struct {
	ID   string `json:"id"`
	Name string `json:"name"`
}

// KplerProduct represents a product with ID and name
type KplerExcelProduct struct {
	ID   string `json:"id"`
	Name string `json:"name"`
}

// KplerFlowsRequest represents the request structure for Kpler flows API
type KplerFlowsRequest struct {
	Platform            string          `json:"platform"`
	Origins            []KplerExcelLocation `json:"origins"`
	Destinations       []KplerExcelLocation `json:"destinations"`
	FromInstallations  []KplerExcelLocation `json:"fromInstallations"`
	ToInstallations    []KplerExcelLocation `json:"toInstallations"`
	FlowDirection      string          `json:"flowDirection"`
	Products           []KplerExcelProduct  `json:"products"`
	Unit               string          `json:"unit"`
	IsProductEstimation bool           `json:"isProductEstimation"`
	IsIntracountry     bool           `json:"isIntracountry"`
	IsIntraRegion      bool           `json:"isIntraRegion"`
	IsWithForecast     bool           `json:"isWithForecast"`
	Granularity        string         `json:"granularity"`
	VesselClassification string       `json:"vesselClassification"`
	VesselsTypes        []string      `json:"vesselsTypes"`
	Split               string        `json:"split"`
	IsFreightView       bool          `json:"isFreightView"`
	IsWithPeriodEndTime bool          `json:"isWithPeriodEndTime"`
	Projection          string        `json:"projection"`
	SelectedPreset      string        `json:"selectedPreset"`
	StartDate           *string       `json:"startDate"`
	EndDate             *string       `json:"endDate"`
}

// ZoneInfo represents a geographical zone in Kpler API
type ZoneInfo struct {
	ID   string `json:"id"`
	Name string `json:"name"`
}

// ProductInfo represents a product in Kpler API
type ProductInfo struct {
	ID   string `json:"id"`
	Name string `json:"name"`
}

// KplerDataPoint represents a single data point with end date and value
type KplerDataPoint struct {
	EndDate string  `json:"end_date"`
	Value   string `json:"val"`
}

// KplerExcelIndexData represents a single index data from Excel
type KplerExcelIndexData struct {
	Title      string            
	Name       string            
	DataPoints []KplerDataPoint  
	Request    string        
}

// KplerFlowsExcelRequest is an alias for KplerFlowsRequest
type KplerFlowsExcelRequest = KplerFlowsRequest

// Process data rows
type DataPoint struct {
	Value string
	Row   int
}